/*
   Copyright (C) 2008-2009 Omat Holding B.V. <info@omat.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include "messageroutewidget.h"

#include <QGridLayout>
#include <QLabel>
#include <QNetworkAccessManager>
#include <QNetworkReply>

#include <KDebug>
#include <KDialog>
#include <KLocale>
#include <KProgressDialog>
#include <kio/job.h>

#include <marble/MarbleWidget.h>
#include <marble/GeoDataPlacemark.h>

using namespace Mailody;
using namespace Marble;
using namespace KIO;

MessageRouteWidget::MessageRouteWidget( QWidget* parent )
        : KDialog( parent )
{
    setButtons( KDialog::Ok );
    setCaption( i18n( "Message route" ) );

    m_marble = new Marble::MarbleWidget( this );
    m_marble->setMapThemeId( "earth/srtm/srtm.dgml" );

    setMainWidget( m_marble );

    setMinimumSize( 400,400 );
    KConfigGroup config = KGlobal::config()->group( "MessageRoute" );
    restoreDialogSize( config );
}

void MessageRouteWidget::accept()
{
    KConfigGroup config = KGlobal::config()->group( "MessageRoute" );
    saveDialogSize( config );
    config.sync();
    done( QDialog::Accepted );
}

void MessageRouteWidget::setIPs( const QStringList& ips )
{
    m_ips = ips;
    m_progress = new KProgressDialog( this, i18n( "Fetching locations..." ),
                                      i18n( "Fetching the location, please wait..." ) );
    m_progress->progressBar()->setMaximum( m_ips.count() );

    foreach( const QString& ip, m_ips ) {
        getIPMethod1( ip );
    }
}

void MessageRouteWidget::getIPMethod1( const QString& ip )
{
    ip2geoHostip* job = new ip2geoHostip( ip, this );
    connect( job, SIGNAL( result( GeoInfo ) ),
             this, SLOT( slotReply1( GeoInfo ) ) );
}

void MessageRouteWidget::getIPMethod2( const QString& ip )
{
    ip2geoHostip* job = new ip2geoHostip( ip, this );
    connect( job, SIGNAL( result( GeoInfo ) ),
             this, SLOT( slotReply2( GeoInfo ) ) );
}

void MessageRouteWidget::slotReply1( GeoInfo info )
{
    if ( info.longitude.isEmpty() || info.latitude.isEmpty() ) {
        kDebug() << "Method 1 did not work out for " << info.ip;
        getIPMethod2( info.ip );
        return;
    }

    m_ipinfo[ info.ip ] = info;

    m_progress->progressBar()->setValue( m_ipinfo.count() );
    kDebug() << "we now have " << m_ipinfo.count() << " / " << m_ips.count();
    if ( m_ipinfo.count() == m_ips.count() )
        addIPsToWidget();
}

void MessageRouteWidget::slotReply2( GeoInfo info )
{
    if ( info.longitude.isEmpty() || info.latitude.isEmpty() ) {
        kDebug() << "Method 2 gave invalid result. deleting " << info.ip;
        int stepsBeforeFinished =  m_ips.count() -  m_progress->progressBar()->value();

        m_ips.removeAll( info.ip );
        m_progress->progressBar()->setMaximum( m_ips.count() );
        m_progress->progressBar()->setValue( m_ips.count() - stepsBeforeFinished );
    } else {
        m_ipinfo[ info.ip ] = info;
    }

    m_progress->progressBar()->setValue( m_ipinfo.count() );
    kDebug() << "we now have " << m_ipinfo.count() << " / " << m_ips.count();
    if ( m_ipinfo.count() == m_ips.count() )
        addIPsToWidget();
}

void MessageRouteWidget::addIPsToWidget()
{
    // copied from digikam...
    QDomDocument       kmlDocument;
    QDomImplementation impl;
    QDomProcessingInstruction instr = kmlDocument.createProcessingInstruction( "xml","version=\"1.0\" encoding=\"UTF-8\"" );
    kmlDocument.appendChild( instr );
    QDomElement kmlRoot = kmlDocument.createElementNS( "http://earth.google.com/kml/2.1","kml" );
    kmlDocument.appendChild( kmlRoot );

    QDomElement kmlAlbum = addKmlElement( kmlDocument, kmlRoot, "Document" );
    QDomElement kmlName = addKmlTextElement( kmlDocument, kmlAlbum, "name", "Geolocation" );

    int i=m_ips.count();
    QString coordinates;
    foreach( const QString& ip, m_ips ) {
        if ( m_ipinfo[ip].longitude.isEmpty() ) {
            --i;
            continue;
        }
        QDomElement kmlPlacemark = addKmlElement( kmlDocument, kmlAlbum, "Placemark" );
        addKmlTextElement( kmlDocument, kmlPlacemark, "name",
                           QString( "%1.%2 (%3)" ).arg( QString::number( i ), m_ipinfo[ip].city, ip ) );

        QDomElement kmlGeometry  = addKmlElement( kmlDocument, kmlPlacemark, "Point" );
        addKmlTextElement( kmlDocument, kmlGeometry, "coordinates",
                           QString( "%1,%2" ).arg( m_ipinfo[ip].longitude,
                                                   m_ipinfo[ip].latitude ) );
        addKmlTextElement( kmlDocument, kmlGeometry, "altitudeMode", "clampToGround" );
        addKmlTextElement( kmlDocument, kmlGeometry, "extrude", "1" );

        coordinates += QString( "%1,%2 " ).arg( m_ipinfo[ip].longitude,
                                                m_ipinfo[ip].latitude );
        --i;
    }

    QDomElement kmlElement = addKmlElement( kmlDocument, kmlAlbum, "Placemark" );
    addKmlTextElement( kmlDocument, kmlElement, "styleUrl", "#geolocation" );
    QDomElement kmlGeometry  = addKmlElement( kmlDocument, kmlElement, "LineString" );
    addKmlTextElement( kmlDocument, kmlGeometry, "coordinates", coordinates );

    kmlElement = addKmlElement( kmlDocument, kmlAlbum, "StyleMap" );
    kmlElement.setAttribute( "id", "geolocation" );
    QDomElement kmlLine = addKmlElement( kmlDocument, kmlElement, "Pair" );
    addKmlTextElement( kmlDocument, kmlLine, "key", "normal" );
    addKmlTextElement( kmlDocument, kmlLine, "styleUrl", "#normalState" );

    kmlElement = addKmlElement( kmlDocument, kmlAlbum, "Style" );
    kmlElement.setAttribute( "id", "normalState" );
    kmlLine = addKmlElement( kmlDocument, kmlElement, "LineStyle" );
    addKmlTextElement( kmlDocument, kmlLine, "width", "5" );
    addKmlTextElement( kmlDocument, kmlLine, "color", "7F0000FF" );

    /*
    QDomElement kmlTimeStamp = addKmlElement(kmlDocument, kmlPlacemark, "TimeStamp");
    addKmlTextElement(kmlDocument, kmlTimeStamp, "when",
                            (*it).dateTime.toString("yyyy-MM-ddThh:mm:ssZ"));
    */

    m_marble->setHome( m_ipinfo[m_ips.last()].longitude.toFloat(),
                       m_ipinfo[m_ips.last()].latitude.toFloat() );
    m_marble->centerOn( m_ipinfo[m_ips.last()].longitude.toFloat(),
                        m_ipinfo[m_ips.last()].latitude.toFloat() );
    kDebug() << kmlDocument.toString();
    m_marble->addPlacemarkData( kmlDocument.toString() );
}

// copied from digikam...
QDomElement MessageRouteWidget::addKmlElement( QDomDocument &kmlDocument, QDomElement &target, const QString& tag )
{
    QDomElement kmlElement = kmlDocument.createElement( tag );
    target.appendChild( kmlElement );
    return kmlElement;
}

// copied from digikam...
QDomElement MessageRouteWidget::addKmlTextElement( QDomDocument &kmlDocument, QDomElement &target,
        const QString& tag, const QString& text )
{
    QDomElement kmlElement  = kmlDocument.createElement( tag );
    target.appendChild( kmlElement );
    QDomText kmlTextElement = kmlDocument.createTextNode( text );
    kmlElement.appendChild( kmlTextElement );
    return kmlElement;
}


#include "messageroutewidget.moc"
